==> 0.0.1.tar.gz: lib/src/widgets/tables/table_utils.dart <==
import 'package:verttice_ui/enums/filter_type.dart';
import 'package:verttice_ui/models/filters/filter_field.dart';
import "package:verttice_ui/utils/data_class_utils.dart";
import "package:verttice_ui/utils/dates.dart";
import "package:verttice_ui/src/form_factory/form_factory.dart";
import "package:verttice_ui/src/form_factory/form_fields.dart";
import 'package:flutter/material.dart';

bool isNumericColumn(FormItem item) {
  return (item.field is DateFF ||
      item.field is ReadonlyDateFF ||
      item.field is TimeFF ||
      item.field is ReadonlyTimeFF ||
      item.field is ReadonlyDateTimeFF ||
      item.field is NumberFF);
}

isTextField(FormItem item) {
  return (item.field is TextFF ||
      item.field is TextBoxFF ||
      item.field is RichTextFF ||
      item.field is SelectFF ||
      item.field is TextListFF ||
      item.field is ExpandMultiSelectFF ||
      item.field is ExpandSelectFF ||
      item.field is GooglePlacesFF ||
      item.field is EmailFF ||
      item.field is PhoneNumberFF ||
      item.field is RadioGroupFF ||
      item.field is EmailListFF ||
      item.field is PhoneNumberListFF ||
      item.field is ReadonlyTextFF);
}

String fieldFromPathToField(
  List<String> pathToField,
  Map<String, dynamic> map,
) {
  dynamic nestedElement = map;
  String finalValue = "";
  for (var entry in pathToField.asMap().entries) {
    if (nestedElement is Map) {
      nestedElement = nestedElement[entry.value];
    } else {
      finalValue = nestedElement?.toString() ?? "";
      break;
    }
    if ((entry.key + 1) == pathToField.length) {
      finalValue = nestedElement?.toString() ?? "";
      break;
    }
  }

  return finalValue;
}

bool applayTableFilter({
  required FilterField filter,
  required dynamic data,
}) {
  if (filter.value == null) {
    return true;
  } else if (data == null) {
    return false;
  } else if (filter.filterType == FilterTypes.string) {
    if (filter.value is String) {
      if (data.toString().toLowerCase().contains(filter.value.toLowerCase())) {
        return true;
      }
    } else if (filter.value is List && filter.value.isNotEmpty) {
      for (var valueList in (filter.value as List)) {
        if (valueList != null && valueList != '') {
          if (data.toString().toLowerCase().contains(valueList.toLowerCase())) {
            return true;
          }
        }
      }
    }
  } else if (filter.filterType == FilterTypes.number) {
    if (filter.value is Map &&
        filter.value['smallNumber'] != null &&
        filter.value['smallNumber'] is int &&
        filter.value['bigNumber'] != null &&
        filter.value['bigNumber'] is int &&
        (data is int) &&
        (data > filter.value["smallNumber"]) &&
        (data < filter.value["bigNumber"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['smallNumber'] != null &&
        filter.value['smallNumber'] is int &&
        (data is int) &&
        (data > filter.value["smallNumber"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['bigNumber'] != null &&
        filter.value['bigNumber'] is int &&
        (data is int) &&
        (data < filter.value["bigNumber"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypes.numberRange) {
  } else if (filter.filterType == FilterTypes.date) {
    if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is int) &&
        (data > filter.value["startDate"]) &&
        (data < filter.value["endDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        (data is int) &&
        (data > filter.value["startDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is int) &&
        (data < filter.value["endDate"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypes.dateRange) {
    if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is Map) &&
        (data['startDate'] is int) &&
        (data['endDate'] is int) &&
        (data['startDate'] > filter.value["startDate"]) &&
        (data['endDate'] < filter.value["endDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        (data is Map) &&
        (data['startDate'] is int) &&
        (data['startDate'] > filter.value["startDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is Map) &&
        (data['endDate'] is int) &&
        (data['endDate'] < filter.value["endDate"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypes.time) {
    if (filter.value is Map &&
        filter.value['startTime'] != null &&
        filter.value['startTime'] is int &&
        filter.value['endTime'] != null &&
        filter.value['endTime'] is int &&
        (data is int) &&
        (data > filter.value["startTime"]) &&
        (data < filter.value["endTime"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['startTime'] != null &&
        filter.value['startTime'] is int &&
        (data is int) &&
        (data > filter.value["startTime"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['endTime'] != null &&
        filter.value['endTime'] is int &&
        (data is int) &&
        (data < filter.value["endTime"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypes.timeRange) {
  } else if (filter.filterType == FilterTypes.bool) {
  } else if (filter.filterType == FilterTypes.geo) {}

  return false;
}

class VisualData {
  final Widget widget;
  final String value;

  VisualData({
    required this.widget,
    required this.value,
  });
}

VisualData visualDataProcessor(
  FormItem item,
  Map<String, dynamic> row, {
  bool lookForPathToFieldOrItemBuilder = true,
}) {
  dynamic value = row[item.field.name];

  if (lookForPathToFieldOrItemBuilder && item.field.dataBuilder != null) {
    value = item.field.dataBuilder!(row);
  } else if (lookForPathToFieldOrItemBuilder &&
      item.field.pathToField != null &&
      item.field.pathToField!.isNotEmpty) {
    value = fieldFromPathToField(
      item.field.pathToField!,
      row,
    );
  }

  String returnValue = "";
  if (item.field.showOnTable && item.field.name != "") {
    if (item.field is ColorSelectorFF) {
      SelectorColors colorSelectorValue = SelectorColors.grey;
      for (var selectorColor in SelectorColors.values) {
        if (selectorColor.name == value) {
          colorSelectorValue = selectorColor;
        }
      }
      late Color color;
      switch (colorSelectorValue) {
        case SelectorColors.green:
          color = Colors.green;
          break;
        case SelectorColors.red:
          color = Colors.red;
          break;

        case SelectorColors.orange:
          color = Colors.orange;
          break;

        case SelectorColors.grey:
          color = Colors.grey;
          break;

        case SelectorColors.yellow:
          color = Colors.yellow;
          break;
      }

      return VisualData(
        value: colorSelectorValue.name,
        widget: Container(
          color: color,
        ),
      );
    } else if (item.field is DateFF ||
        item.field is ReadonlyDateFF ||
        item.field is TimeFF ||
        item.field is ReadonlyTimeFF ||
        item.field is ReadonlyDateTimeFF ||
        item.field is TextFF ||
        item.field is TextBoxFF ||
        item.field is NumberFF ||
        item.field is ReadonlyNumberFF ||
        item.field is DateRangeFF ||
        item.field is ReadonlyDateRangeFF ||
        item.field is DateRangeFF ||
        item.field is DateTimeRangeFF ||
        item.field is ReadonlyDateTimeRangeFF) {
      String formattedValue = "";
      if (item.field is NumberFF || item.field is ReadonlyNumberFF) {
        formattedValue =
            DataClassUtils.dynamicToDouble(value)!.toStringAsFixed(2);
      } else if (item.field is DateFF || item.field is ReadonlyDateFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: false);
      } else if (item.field is TimeFF || item.field is ReadonlyTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: false,
            time: true);
      } else if (item.field is ReadonlyDateTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: true);
      } else if (item.field is DateRangeFF ||
          item.field is ReadonlyDateRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!, time: false) : ""} - ${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!, time: false) : ""}";
      } else if (item.field is DateTimeRangeFF ||
          item.field is ReadonlyDateTimeRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!) : ""} - \n${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!) : ""}";
      } else {
        formattedValue = DataClassUtils.dynamicToString(value);
      }

      returnValue = formattedValue;
    } else {
      returnValue = DataClassUtils.dynamicToString(value);
    }
  }

  return VisualData(
    value: returnValue,
    widget: Text(
      returnValue,
      style: const TextStyle(
        fontSize: 12,
      ),
    ),
  );
}

==> 0.0.13.tar.gz: lib/src/widgets/tables/table_utils.dart <==
import 'package:verttice_logic/verttice_logic.dart';
import "package:verttice_ui/src/widgets/form_factory/form_factory.dart";
import "package:verttice_ui/src/widgets/form_factory/form_fields.dart";
import 'package:flutter/material.dart';

bool isNumericColumn(FormItem item) {
  return (item.field is DateFF ||
      item.field is ReadonlyDateFF ||
      item.field is TimeFF ||
      item.field is ReadonlyTimeFF ||
      item.field is ReadonlyDateTimeFF ||
      item.field is NumberFF);
}

isTextField(FormItem item) {
  return (item.field is TextFF ||
      item.field is TextBoxFF ||
      item.field is RichTextFF ||
      item.field is SelectFF ||
      item.field is TextListFF ||
      item.field is ExpandMultiSelectFF ||
      item.field is ExpandSelectFF ||
      item.field is GooglePlacesFF ||
      item.field is EmailFF ||
      item.field is PhoneNumberFF ||
      item.field is RadioGroupFF ||
      item.field is EmailListFF ||
      item.field is PhoneNumberListFF ||
      item.field is ReadonlyTextFF);
}

String fieldFromPathToField(
  List<String> pathToField,
  Map<String, dynamic> map,
) {
  dynamic nestedElement = map;
  String finalValue = "";
  for (var entry in pathToField.asMap().entries) {
    if (nestedElement is Map) {
      nestedElement = nestedElement[entry.value];
    } else {
      finalValue = nestedElement?.toString() ?? "";
      break;
    }
    if ((entry.key + 1) == pathToField.length) {
      finalValue = nestedElement?.toString() ?? "";
      break;
    }
  }

  return finalValue;
}

bool applayTableFilter({
  required FilterField filter,
  required dynamic data,
}) {
  if (filter.value == null) {
    return true;
  } else if (data == null) {
    return false;
  } else if (filter.filterType == FilterTypesFF.string) {
    if (filter.value is String) {
      if (data.toString().toLowerCase().contains(filter.value.toLowerCase())) {
        return true;
      }
    } else if (filter.value is List && filter.value.isNotEmpty) {
      for (var valueList in (filter.value as List)) {
        if (valueList != null && valueList != '') {
          if (data.toString().toLowerCase().contains(valueList.toLowerCase())) {
            return true;
          }
        }
      }
    }
  } else if (filter.filterType == FilterTypesFF.number) {
    if (filter.value is Map &&
        filter.value['smallNumber'] != null &&
        filter.value['smallNumber'] is int &&
        filter.value['bigNumber'] != null &&
        filter.value['bigNumber'] is int &&
        (data is int) &&
        (data > filter.value["smallNumber"]) &&
        (data < filter.value["bigNumber"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['smallNumber'] != null &&
        filter.value['smallNumber'] is int &&
        (data is int) &&
        (data > filter.value["smallNumber"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['bigNumber'] != null &&
        filter.value['bigNumber'] is int &&
        (data is int) &&
        (data < filter.value["bigNumber"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypesFF.numberRange) {
  } else if (filter.filterType == FilterTypesFF.date) {
    if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is int) &&
        (data > filter.value["startDate"]) &&
        (data < filter.value["endDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        (data is int) &&
        (data > filter.value["startDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is int) &&
        (data < filter.value["endDate"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypesFF.dateRange) {
    if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is Map) &&
        (data['startDate'] is int) &&
        (data['endDate'] is int) &&
        (data['startDate'] > filter.value["startDate"]) &&
        (data['endDate'] < filter.value["endDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['startDate'] != null &&
        filter.value['startDate'] is int &&
        (data is Map) &&
        (data['startDate'] is int) &&
        (data['startDate'] > filter.value["startDate"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['endDate'] != null &&
        filter.value['endDate'] is int &&
        (data is Map) &&
        (data['endDate'] is int) &&
        (data['endDate'] < filter.value["endDate"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypesFF.time) {
    if (filter.value is Map &&
        filter.value['startTime'] != null &&
        filter.value['startTime'] is int &&
        filter.value['endTime'] != null &&
        filter.value['endTime'] is int &&
        (data is int) &&
        (data > filter.value["startTime"]) &&
        (data < filter.value["endTime"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['startTime'] != null &&
        filter.value['startTime'] is int &&
        (data is int) &&
        (data > filter.value["startTime"])) {
      return true;
    } else if (filter.value is Map &&
        filter.value['endTime'] != null &&
        filter.value['endTime'] is int &&
        (data is int) &&
        (data < filter.value["endTime"])) {
      return true;
    }
  } else if (filter.filterType == FilterTypesFF.timeRange) {
  } else if (filter.filterType == FilterTypesFF.bool) {
  } else if (filter.filterType == FilterTypesFF.geo) {}

  return false;
}

class VisualData {
  final Widget widget;
  final String value;

  VisualData({
    required this.widget,
    required this.value,
  });
}

VisualData visualDataProcessor(
  FormItem item,
  Map<String, dynamic> row, {
  bool lookForPathToFieldOrItemBuilder = true,
}) {
  dynamic value = row[item.field.name];

  if (lookForPathToFieldOrItemBuilder && item.field.dataBuilder != null) {
    value = item.field.dataBuilder!(row);
  } else if (lookForPathToFieldOrItemBuilder &&
      item.field.pathToField != null &&
      item.field.pathToField!.isNotEmpty) {
    value = fieldFromPathToField(
      item.field.pathToField!,
      row,
    );
  }

  String returnValue = "";
  if (item.field.showOnTable && item.field.name != "") {
    if (item.field is ColorSelectorFF) {
      SelectorColors colorSelectorValue = SelectorColors.grey;
      for (var selectorColor in SelectorColors.values) {
        if (selectorColor.name == value) {
          colorSelectorValue = selectorColor;
        }
      }
      late Color color;
      switch (colorSelectorValue) {
        case SelectorColors.green:
          color = Colors.green;
          break;
        case SelectorColors.red:
          color = Colors.red;
          break;

        case SelectorColors.orange:
          color = Colors.orange;
          break;

        case SelectorColors.grey:
          color = Colors.grey;
          break;

        case SelectorColors.yellow:
          color = Colors.yellow;
          break;
      }

      return VisualData(
        value: colorSelectorValue.name,
        widget: Container(
          color: color,
        ),
      );
    } else if (item.field is DateFF ||
        item.field is ReadonlyDateFF ||
        item.field is TimeFF ||
        item.field is ReadonlyTimeFF ||
        item.field is ReadonlyDateTimeFF ||
        item.field is TextFF ||
        item.field is TextBoxFF ||
        item.field is NumberFF ||
        item.field is ReadonlyNumberFF ||
        item.field is DateRangeFF ||
        item.field is ReadonlyDateRangeFF ||
        item.field is DateRangeFF ||
        item.field is DateTimeRangeFF ||
        item.field is ReadonlyDateTimeRangeFF) {
      String formattedValue = "";
      if (item.field is NumberFF || item.field is ReadonlyNumberFF) {
        formattedValue =
            DataClassUtils.dynamicToDouble(value)!.toStringAsFixed(2);
      } else if (item.field is DateFF || item.field is ReadonlyDateFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: false);
      } else if (item.field is TimeFF || item.field is ReadonlyTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: false,
            time: true);
      } else if (item.field is ReadonlyDateTimeFF) {
        formattedValue = Dates.formatDataAndTime(
            miliseconds: DataClassUtils.dynamicToInt(value, defultValue: 0)!,
            date: true,
            time: true);
      } else if (item.field is DateRangeFF ||
          item.field is ReadonlyDateRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!, time: false) : ""} - ${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!, time: false) : ""}";
      } else if (item.field is DateTimeRangeFF ||
          item.field is ReadonlyDateTimeRangeFF) {
        formattedValue = value == null || value is! Map
            ? ""
            : "${value["startDate"] != null && value["startDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["startDate"], defultValue: 0)!) : ""} - \n${value["endDate"] != null && value["endDate"] is int ? Dates.formatDataAndTime(miliseconds: DataClassUtils.dynamicToInt(value["endDate"], defultValue: 0)!) : ""}";
      } else {
        formattedValue = DataClassUtils.dynamicToString(value);
      }

      returnValue = formattedValue;
    } else {
      returnValue = DataClassUtils.dynamicToString(value);
    }
  }

  return VisualData(
    value: returnValue,
    widget: Text(
      returnValue,
      style: const TextStyle(
        fontSize: 12,
      ),
    ),
  );
}
